#ifndef CH2PARSER_HPP_
#define CH2PARSER_HPP_

#include <cstdlib>
#include <cassert>
#include <vector>

namespace Ch2Parser {

enum Token {
    token_eof,
    token_BEID,
    token_BE_PICT_URL,
    token_DATE,
    token_DAY,
    token_ID,
    token_SEPARATOR,
    token_TEXT,
    token_URL,
};

template < class T, int StackSize >
class Stack {
public:
	Stack(){ gap_ = 0; }
	~Stack(){}
	
	void reset_tmp()
	{
		gap_ = stack_.size();
		tmp_.clear();
	}

	void commit_tmp()
	{
		// may throw
		stack_.reserve( gap_ + tmp_.size() );

		// expect not to throw
stack_.erase( stack_.begin() + gap_, stack_.end() );
		stack_.insert( stack_.end(), tmp_.begin(), tmp_.end() );
	}
	bool push( const T& f )
	{
		if( StackSize != 0 && StackSize <= stack_.size() + tmp_.size() ) {
			return false;
		}
		tmp_.push_back( f );
		return true;
	}

	void pop( size_t n )
	{
		if( tmp_.size() < n ) {
			n -= tmp_.size();
			tmp_.clear();
			gap_ -= n;
		} else {
			tmp_.erase( tmp_.end() - n, tmp_.end() );
		}
	}

	const T& top()
	{
		if( !tmp_.empty() ) {
			return tmp_.back();
		} else {
			return stack_[ gap_ - 1 ];
		}
	}

	const T& get_arg( size_t base, size_t index )
	{
		size_t n = tmp_.size();
		if( base - index <= n ) {
			return tmp_[ n - ( base - index ) ];
		} else {
			return stack_[ gap_ - ( base - n ) + index ];
		}
	}

	void clear()
	{
		stack_.clear();
	}

private:
	std::vector< T > stack_;
	std::vector< T > tmp_;
	size_t gap_;

};

template < class Value, class SemanticAction, int StackSize = 0 >
class Parser {
public:
	typedef Token token_type;
	typedef Value value_type;

public:
	Parser( SemanticAction& sa ) : sa_( sa ) { reset(); }

	void reset()
	{
		error_ = false;
		accepted_ = false;
		clear_stack();
		reset_tmp_stack();
		if( push_stack( &Parser::state_0, &Parser::gotof_0, value_type() ) ) {
			commit_tmp_stack();
		} else {
			sa_.stack_overflow();
			error_ = true;
		}
	}

	bool post( token_type token, const value_type& value )
	{
		assert( !error_ );
		reset_tmp_stack();
		while( (this->*(stack_top()->state) )( token, value ) ); // may throw
		if( !error_ ) {
			commit_tmp_stack();
		}
		return accepted_ || error_;
	}

	bool accept( value_type& v )
	{
		assert( accepted_ );
		if( error_ ) { return false; }
		v = accepted_value_;
		return true;
	}

	bool error() { return error_; }

private:
	typedef Parser< Value, SemanticAction, StackSize > self_type;
	typedef bool ( self_type::*state_type )( token_type, const value_type& );
	typedef bool ( self_type::*gotof_type )( int, const value_type& );

	bool            accepted_;
	bool            error_;
	value_type      accepted_value_;

	SemanticAction& sa_;

	struct stack_frame {
		state_type state;
		gotof_type gotof;
		value_type value;

		stack_frame( state_type s, gotof_type g, const value_type& v )
		    : state( s ), gotof( g ), value( v ) {}
	};

	Stack< stack_frame, StackSize > stack_;
	bool push_stack( state_type s, gotof_type g, const value_type& v )
	{
		bool f = stack_.push( stack_frame( s, g, v ) );
		assert( !error_ );
		if( !f ) {
			error_ = true;
			sa_.stack_overflow();
		}
		return f;
	}

	void pop_stack( size_t n )
	{
		stack_.pop( n );
	}

	const stack_frame* stack_top()
	{
		return &stack_.top();
	}

	const value_type& get_arg( size_t base, size_t index )
	{
		return stack_.get_arg( base, index ).value;
	}

	void clear_stack()
	{
		stack_.clear();
	}

	void reset_tmp_stack()
	{
		stack_.reset_tmp();
	}

	void commit_tmp_stack()
	{
		stack_.commit_tmp();
	}

	bool call_0_setDayAndDate( int nonterminal_index, int base, int arg_index0, int arg_index1 )
	{
		wxString arg0; sa_.downcast( arg0, get_arg( base, arg_index0 ) );
		wxString arg1; sa_.downcast( arg1, get_arg( base, arg_index1 ) );
		int r = sa_.setDayAndDate( arg0, arg1 );
		value_type v; sa_.upcast( v, r );
		pop_stack( base );
		return (this->*(stack_top()->gotof))( nonterminal_index, v );
	}

	bool call_0_setIds( int nonterminal_index, int base, int arg_index0 )
	{
		wxString arg0; sa_.downcast( arg0, get_arg( base, arg_index0 ) );
		int r = sa_.setIds( arg0 );
		value_type v; sa_.upcast( v, r );
		pop_stack( base );
		return (this->*(stack_top()->gotof))( nonterminal_index, v );
	}

	bool call_1_setIds( int nonterminal_index, int base, int arg_index0, int arg_index1 )
	{
		wxString arg0; sa_.downcast( arg0, get_arg( base, arg_index0 ) );
		wxString arg1; sa_.downcast( arg1, get_arg( base, arg_index1 ) );
		int r = sa_.setIds( arg0, arg1 );
		value_type v; sa_.upcast( v, r );
		pop_stack( base );
		return (this->*(stack_top()->gotof))( nonterminal_index, v );
	}

	bool call_0_setTitleMode( int nonterminal_index, int base )
	{
		int r = sa_.setTitleMode(  );
		value_type v; sa_.upcast( v, r );
		pop_stack( base );
		return (this->*(stack_top()->gotof))( nonterminal_index, v );
	}

	bool call_0_setText( int nonterminal_index, int base, int arg_index0 )
	{
		wxString arg0; sa_.downcast( arg0, get_arg( base, arg_index0 ) );
		int r = sa_.setText( arg0 );
		value_type v; sa_.upcast( v, r );
		pop_stack( base );
		return (this->*(stack_top()->gotof))( nonterminal_index, v );
	}

	bool gotof_0( int nonterminal_index, const value_type& v )
	{
		switch( nonterminal_index ) {
		case 0: return push_stack( &Parser::state_1, &Parser::gotof_1, v );
		case 3: return push_stack( &Parser::state_2, &Parser::gotof_2, v );
		case 2: return push_stack( &Parser::state_3, &Parser::gotof_3, v );
		case 1: return push_stack( &Parser::state_5, &Parser::gotof_5, v );
		case 4: return push_stack( &Parser::state_25, &Parser::gotof_25, v );
		default: assert(0); return false;
		}
	}

	bool state_0( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_TEXT:
			// shift
			push_stack( &Parser::state_32, &Parser::gotof_32, value );
			return false;
		case token_URL:
			// shift
			push_stack( &Parser::state_31, &Parser::gotof_31, value );
			return false;
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_1( int nonterminal_index, const value_type& v )
	{
		assert(0);
		return true;
	}

	bool state_1( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_eof:
			// accept
			// run_semantic_action();
			accepted_ = true;
			accepted_value_  = get_arg( 1, 0 );
			return false;
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_2( int nonterminal_index, const value_type& v )
	{
		switch( nonterminal_index ) {
		case 2: return push_stack( &Parser::state_4, &Parser::gotof_4, v );
		case 1: return push_stack( &Parser::state_5, &Parser::gotof_5, v );
		case 4: return push_stack( &Parser::state_25, &Parser::gotof_25, v );
		default: assert(0); return false;
		}
	}

	bool state_2( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_eof:
			// reduce
				// run_semantic_action();
				pop_stack( 1 );
				return (this->*(stack_top()->gotof))( 0, value_type() );
		case token_TEXT:
			// shift
			push_stack( &Parser::state_32, &Parser::gotof_32, value );
			return false;
		case token_URL:
			// shift
			push_stack( &Parser::state_31, &Parser::gotof_31, value );
			return false;
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_3( int nonterminal_index, const value_type& v )
	{
		assert(0);
		return true;
	}

	bool state_3( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_eof:
		case token_TEXT:
		case token_URL:
			return call_0_setTitleMode( 3, 1 );
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_4( int nonterminal_index, const value_type& v )
	{
		assert(0);
		return true;
	}

	bool state_4( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_eof:
			// reduce
				// run_semantic_action();
				pop_stack( 2 );
				return (this->*(stack_top()->gotof))( 3, value_type() );
		case token_TEXT:
			// reduce
				// run_semantic_action();
				pop_stack( 2 );
				return (this->*(stack_top()->gotof))( 3, value_type() );
		case token_URL:
			// reduce
				// run_semantic_action();
				pop_stack( 2 );
				return (this->*(stack_top()->gotof))( 3, value_type() );
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_5( int nonterminal_index, const value_type& v )
	{
		assert(0);
		return true;
	}

	bool state_5( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_BEID:
			// shift
			push_stack( &Parser::state_13, &Parser::gotof_13, value );
			return false;
		case token_ID:
			// shift
			push_stack( &Parser::state_9, &Parser::gotof_9, value );
			return false;
		case token_SEPARATOR:
			// shift
			push_stack( &Parser::state_6, &Parser::gotof_6, value );
			return false;
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_6( int nonterminal_index, const value_type& v )
	{
		switch( nonterminal_index ) {
		case 4: return push_stack( &Parser::state_7, &Parser::gotof_7, v );
		default: assert(0); return false;
		}
	}

	bool state_6( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_TEXT:
			// shift
			push_stack( &Parser::state_32, &Parser::gotof_32, value );
			return false;
		case token_URL:
			// shift
			push_stack( &Parser::state_31, &Parser::gotof_31, value );
			return false;
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_7( int nonterminal_index, const value_type& v )
	{
		assert(0);
		return true;
	}

	bool state_7( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_SEPARATOR:
			// shift
			push_stack( &Parser::state_8, &Parser::gotof_8, value );
			return false;
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_8( int nonterminal_index, const value_type& v )
	{
		switch( nonterminal_index ) {
		case 4: return push_stack( &Parser::state_21, &Parser::gotof_21, v );
		default: assert(0); return false;
		}
	}

	bool state_8( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_eof:
			// reduce
				// run_semantic_action();
				pop_stack( 4 );
				return (this->*(stack_top()->gotof))( 2, value_type() );
		case token_TEXT:
			// shift
			push_stack( &Parser::state_32, &Parser::gotof_32, value );
			return false;
		case token_URL:
			// shift
			push_stack( &Parser::state_31, &Parser::gotof_31, value );
			return false;
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_9( int nonterminal_index, const value_type& v )
	{
		assert(0);
		return true;
	}

	bool state_9( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_BEID:
			// shift
			push_stack( &Parser::state_17, &Parser::gotof_17, value );
			return false;
		case token_SEPARATOR:
			// shift
			push_stack( &Parser::state_10, &Parser::gotof_10, value );
			return false;
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_10( int nonterminal_index, const value_type& v )
	{
		switch( nonterminal_index ) {
		case 4: return push_stack( &Parser::state_11, &Parser::gotof_11, v );
		default: assert(0); return false;
		}
	}

	bool state_10( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_TEXT:
			// shift
			push_stack( &Parser::state_32, &Parser::gotof_32, value );
			return false;
		case token_URL:
			// shift
			push_stack( &Parser::state_31, &Parser::gotof_31, value );
			return false;
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_11( int nonterminal_index, const value_type& v )
	{
		assert(0);
		return true;
	}

	bool state_11( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_SEPARATOR:
			// shift
			push_stack( &Parser::state_12, &Parser::gotof_12, value );
			return false;
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_12( int nonterminal_index, const value_type& v )
	{
		switch( nonterminal_index ) {
		case 4: return push_stack( &Parser::state_22, &Parser::gotof_22, v );
		default: assert(0); return false;
		}
	}

	bool state_12( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_TEXT:
			// shift
			push_stack( &Parser::state_32, &Parser::gotof_32, value );
			return false;
		case token_URL:
			// shift
			push_stack( &Parser::state_31, &Parser::gotof_31, value );
			return false;
		case token_eof:
			return call_0_setIds( 2, 5, 1 );
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_13( int nonterminal_index, const value_type& v )
	{
		assert(0);
		return true;
	}

	bool state_13( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_SEPARATOR:
			// shift
			push_stack( &Parser::state_14, &Parser::gotof_14, value );
			return false;
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_14( int nonterminal_index, const value_type& v )
	{
		switch( nonterminal_index ) {
		case 4: return push_stack( &Parser::state_15, &Parser::gotof_15, v );
		default: assert(0); return false;
		}
	}

	bool state_14( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_TEXT:
			// shift
			push_stack( &Parser::state_32, &Parser::gotof_32, value );
			return false;
		case token_URL:
			// shift
			push_stack( &Parser::state_31, &Parser::gotof_31, value );
			return false;
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_15( int nonterminal_index, const value_type& v )
	{
		assert(0);
		return true;
	}

	bool state_15( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_SEPARATOR:
			// shift
			push_stack( &Parser::state_16, &Parser::gotof_16, value );
			return false;
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_16( int nonterminal_index, const value_type& v )
	{
		switch( nonterminal_index ) {
		case 4: return push_stack( &Parser::state_23, &Parser::gotof_23, v );
		default: assert(0); return false;
		}
	}

	bool state_16( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_TEXT:
			// shift
			push_stack( &Parser::state_32, &Parser::gotof_32, value );
			return false;
		case token_URL:
			// shift
			push_stack( &Parser::state_31, &Parser::gotof_31, value );
			return false;
		case token_eof:
			return call_0_setIds( 2, 5, 1 );
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_17( int nonterminal_index, const value_type& v )
	{
		assert(0);
		return true;
	}

	bool state_17( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_SEPARATOR:
			// shift
			push_stack( &Parser::state_18, &Parser::gotof_18, value );
			return false;
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_18( int nonterminal_index, const value_type& v )
	{
		switch( nonterminal_index ) {
		case 4: return push_stack( &Parser::state_19, &Parser::gotof_19, v );
		default: assert(0); return false;
		}
	}

	bool state_18( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_TEXT:
			// shift
			push_stack( &Parser::state_32, &Parser::gotof_32, value );
			return false;
		case token_URL:
			// shift
			push_stack( &Parser::state_31, &Parser::gotof_31, value );
			return false;
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_19( int nonterminal_index, const value_type& v )
	{
		assert(0);
		return true;
	}

	bool state_19( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_SEPARATOR:
			// shift
			push_stack( &Parser::state_20, &Parser::gotof_20, value );
			return false;
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_20( int nonterminal_index, const value_type& v )
	{
		switch( nonterminal_index ) {
		case 4: return push_stack( &Parser::state_24, &Parser::gotof_24, v );
		default: assert(0); return false;
		}
	}

	bool state_20( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_TEXT:
			// shift
			push_stack( &Parser::state_32, &Parser::gotof_32, value );
			return false;
		case token_URL:
			// shift
			push_stack( &Parser::state_31, &Parser::gotof_31, value );
			return false;
		case token_eof:
			return call_1_setIds( 2, 6, 1, 2 );
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_21( int nonterminal_index, const value_type& v )
	{
		assert(0);
		return true;
	}

	bool state_21( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_eof:
			// reduce
				// run_semantic_action();
				pop_stack( 5 );
				return (this->*(stack_top()->gotof))( 2, value_type() );
		case token_TEXT:
			// reduce
				// run_semantic_action();
				pop_stack( 5 );
				return (this->*(stack_top()->gotof))( 2, value_type() );
		case token_URL:
			// reduce
				// run_semantic_action();
				pop_stack( 5 );
				return (this->*(stack_top()->gotof))( 2, value_type() );
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_22( int nonterminal_index, const value_type& v )
	{
		assert(0);
		return true;
	}

	bool state_22( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_eof:
		case token_TEXT:
		case token_URL:
			return call_0_setIds( 2, 6, 1 );
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_23( int nonterminal_index, const value_type& v )
	{
		assert(0);
		return true;
	}

	bool state_23( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_eof:
		case token_TEXT:
		case token_URL:
			return call_0_setIds( 2, 6, 1 );
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_24( int nonterminal_index, const value_type& v )
	{
		assert(0);
		return true;
	}

	bool state_24( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_eof:
		case token_TEXT:
		case token_URL:
			return call_1_setIds( 2, 7, 1, 2 );
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_25( int nonterminal_index, const value_type& v )
	{
		assert(0);
		return true;
	}

	bool state_25( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_SEPARATOR:
			// shift
			push_stack( &Parser::state_26, &Parser::gotof_26, value );
			return false;
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_26( int nonterminal_index, const value_type& v )
	{
		switch( nonterminal_index ) {
		case 4: return push_stack( &Parser::state_27, &Parser::gotof_27, v );
		default: assert(0); return false;
		}
	}

	bool state_26( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_TEXT:
			// shift
			push_stack( &Parser::state_32, &Parser::gotof_32, value );
			return false;
		case token_URL:
			// shift
			push_stack( &Parser::state_31, &Parser::gotof_31, value );
			return false;
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_27( int nonterminal_index, const value_type& v )
	{
		assert(0);
		return true;
	}

	bool state_27( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_SEPARATOR:
			// shift
			push_stack( &Parser::state_28, &Parser::gotof_28, value );
			return false;
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_28( int nonterminal_index, const value_type& v )
	{
		assert(0);
		return true;
	}

	bool state_28( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_DAY:
			// shift
			push_stack( &Parser::state_29, &Parser::gotof_29, value );
			return false;
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_29( int nonterminal_index, const value_type& v )
	{
		assert(0);
		return true;
	}

	bool state_29( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_DATE:
			// shift
			push_stack( &Parser::state_30, &Parser::gotof_30, value );
			return false;
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_30( int nonterminal_index, const value_type& v )
	{
		assert(0);
		return true;
	}

	bool state_30( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_BEID:
		case token_ID:
		case token_SEPARATOR:
			return call_0_setDayAndDate( 1, 6, 4, 5 );
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_31( int nonterminal_index, const value_type& v )
	{
		assert(0);
		return true;
	}

	bool state_31( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_eof:
		case token_SEPARATOR:
		case token_TEXT:
		case token_URL:
			return call_0_setText( 4, 1, 0 );
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

	bool gotof_32( int nonterminal_index, const value_type& v )
	{
		assert(0);
		return true;
	}

	bool state_32( token_type token, const value_type& value )
	{
		switch( token ) {
		case token_eof:
		case token_SEPARATOR:
		case token_TEXT:
		case token_URL:
			return call_0_setText( 4, 1, 0 );
		default:
			sa_.syntax_error();
			error_ = true;
			return false;
		}
	}

};

} // namespace Ch2Parser

#endif // #ifndef CH2PARSER_HPP_
